require 'jafry/accessor'

module Jafry
  # Module for saving and changing scheme's configs
  # @since 0.2.0
  module Configurator

    extend Accessor

    @@schemes_configs = []

    mattrs :schemes_configs

    class << self
      # Setups config for scheme
      #
      # @param scheme_name [String] Scheme name
      # @param [Hash] config Config

      def set_config(scheme_name, config={})
        config = default if config.empty?
        scheme_config = init_or_update(scheme_name, config)
        self.schemes_configs << scheme_config
        scheme_config
      end

      # Creates new or updates existed config
      #
      # @param scheme_name [String] Scheme name
      # @param [Hash] config Config to set/update

      def init_or_update(scheme_name, config)
        return find_config(scheme_name)[:config].merge!(config) if find_config(scheme_name)
        {scheme: scheme_name, config: config}
      end

      # Finds config by scheme name
      #
      # @param scheme [String] Scheme name
      # @return [Hash] Config

      def get_config(scheme)
        find_config(scheme)[:config]
      end

      # Returns scheme by name
      #
      # @param scheme_name [String] Scheme name
      # @return [Hash] Hash of scheme name and config

      def find_config(scheme_name)
        self.schemes_configs.select {|item| item[:scheme] == scheme_name}.last
      end

      # Specifies default config
      #
      # @return [Hash] Default config

      def default
        {id_type: Jafry.id_type, id_wrapper: Jafry.id_wrapper}
      end
    end
  end
end
