require 'jafry/accessor'
require 'jafry/configurator'

module Jafry
  # Module for automaticaly creatings schemes ids
  # @since 0.2.0
  module Identificator

    extend Accessor

    @@schemes = []

    mattrs :schemes

    class << self
      # Creates identification record about scheme
      #
      # @param scheme_name [String] Scheme name
      # @return [Hash] Identification information about scheme

      def register_scheme(scheme_name)
        config = Jafry::Configurator.get_config(scheme_name)
        config = Jafry::Configurator.set_config(scheme_name) unless config
        scheme = build_type(config[:id_type]).scheme_scaffold(scheme_name)
        self.schemes << scheme
        scheme
      end

      # Finds registered scheme
      #
      # @param scheme_name [String] Scheme name
      # @return [Hash] Identification information about scheme

      def find_scheme(scheme_name)
        self.schemes.select {|item| item[:scheme] == scheme_name}.last
      end

      # Finds registered scheme and if there is no one, creates new
      #
      # @param scheme_name [String] Scheme name
      # @return [Hash] Identification information about scheme

      def find_or_register(scheme_name)
        return find_scheme(scheme_name) if find_scheme(scheme_name)
        register_scheme(scheme_name)
      end

      # Modifies scheme with current id
      #
      # @param scheme [Scheme] Scheme instance
      # @return [Scheme] Scheme with id

      def set_id_for_scheme(scheme)
        scheme.merge!(generate_id(scheme))
      end

      # Creates new scheme id type
      #
      # @param type [String] Id type
      # @return [Type] Type of id

      def build_type(type)
        const_get("Jafry::Identificator::#{type.capitalize}").new
      end

      # Generates id
      #
      # @param [Scheme] scheme Scheme instance

      def generate_id(scheme)
        config = Jafry::Configurator.get_config(scheme.self_name)
        build_type(config[:id_type]).id_scaffold(config[:id_wrapper], scheme)
      end

      # Increment id counter by 1
      #
      # @param [Scheme] scheme Scheme instance

      def inc_scheme_counter(scheme)
        find_or_register(scheme.self_name)[:counter] += 1
      end

      # Creates hash id
      #
      # @param [Scheme] scheme Scheme instance

      def hash_counter(scheme)
        find_or_register(scheme.self_name)[:counter] = generate_hash
      end

      # Helper method for generate unique hash
      #
      # @return [String] Unique md5 hash

      def generate_hash
        Digest::MD5.hexdigest([Time.now, rand].join)
      end
    end

    class Number
      # Class for scheme id type 'number'
      # @since 0.2.0

      # Specify hash for basic scheme id type
      #
      # @return [Hash] Hash with basic id

      def scheme_scaffold(scheme_name)
        {scheme: scheme_name, counter: 0}
      end

      # Specify hash for basic scheme id type with custom id wrapper
      #
      # @return [Hash] Hash with basic id

      def id_scaffold(wrapper, scheme)
        {"#{wrapper}" => Identificator::inc_scheme_counter(scheme)}
      end
    end

    class Hash
      # Class for scheme id type 'hash'
      # @since 0.2.0

      # Specify hash for basic scheme id type
      #
      # @return [Hash] Hash with basic id

      def scheme_scaffold(scheme_name)
        {scheme: scheme_name, counter: generate_hash}
      end

      # Specify hash for basic scheme id type with custom id wrapper
      #
      # @return [Hash] Hash with basic id

      def id_scaffold(wrapper, scheme)
        {"#{wrapper}" => Identificator::hash_counter(scheme)}
      end
    end
  end
end

