require 'yajl'

module Jafry
  # Bunch of functions to work with encode/decode JSON and files
  # @since 0.2.0

  module Utilator
    private
    # Method for creating full path to scheme file
    #
    # @private
    # @param scheme [String] Scheme name
    # @return [String] Full path to file

    def build_schemes_path(scheme)
      "#{Jafry.schemes_path}/#{scheme}.json".gsub(/\/{2,}/, '/')
    end

    # Method for check scheme file existance
    #
    # @private
    # @param scheme_name [String] Scheme name
    # @return [Boolean] True or false

    def scheme_file_exists?(scheme_name)
      File.exists?(build_schemes_path(scheme_name))
    end

    # Specify message will be shown when scheme file not exist
    #
    # @private
    # @return [String] Message

    def not_exists_message
      "Please, create correct scheme file first"
    end

    # Parses JSON
    #
    # @private
    # @param [String] json JSON to parse
    # @return [Hash] Parsed json

    def parse_json(json)
      Yajl::Parser.new.parse(json)
    end

    # Encodes JSON
    #
    # @private
    # @param [Hash] hash Hash to convert to JSON
    # @return [String] Parsed json

    def encode_json(hash)
      Yajl::Encoder.encode hash
    end

    # Creates Hash from JSON-based scheme file
    #
    # @private
    # @param [String] file_name Scheme file name
    # @return [Hash] Parsed scheme

    def parse_scheme(file_name)
      begin
        raise not_exists_message unless scheme_file_exists?(file_name)
        parse_json(File.read(build_schemes_path(file_name)))
      rescue Exception => e
        puts e.message
        puts e.backtrace
      end
    end

    # Wraps hash in specified wrapper
    #
    # @private
    # @param [String] wrapper Namespace hash will be wrapped in
    # @param [Hash] hash Hash to wrap
    # @return [Hash] Wrapped hash
    #
    # @example Wrap
    #   wrap('baz', {foo: 'bar'}) #=> {'baz' => {foo: 'bar'}}

    def wrap(wrapper, hash)
      {"#{wrapper}" => hash}
    end
  end
end
